from .wsseheader import WsseToken

from .__version__ import __title__, __description__, __url__, __version__
from .__version__ import  __author__, __author_email__, __license__
from .__version__ import __copyright__





# Python code to demonstrate working of unittest 
import unittest
import WsseHeaders

class TestUtilsMethods(unittest.TestCase): 
		
	def test_import(self):
		'''
		check for initializing as None
		'''
		try:
			a = WsseHeaders.WsseToken("None",None,None)
			self.assertEqual(1, 0)
			print(type(a))
		except Exception as e:
			print(e)
			if type(e) is ValueError:
				self.assertEqual(1, 1)
			else:
				self.assertEqual(1, 0)
			


class TestWSSEMethods(unittest.TestCase): 
	
	def test_check_init(self):
		'''
		check for return type STRING
		'''
		try:
			a = WsseHeaders.WsseToken("Debapriya","yodebu","YmFzZTY0c3R0cmluZw==", True)
			print(type(a))
			self.assertEqual(str(""), a.generateHeaders())
			
		except Exception as e:
			print(e)
			if type(e) is ValueError:
				self.assertEqual(1, 1)
			else:
				self.assertEqual(1, 0)

testSuite = unittest.TestSuite()


if __name__ == '__main__': 
	unittest.main()


__title__ = 'WsseHeaders'
__description__ = 'Python WsseHeaders generation Library for Humans.'
__url__ = 'https://abc.xyz'
__version__ = '0.0.1'
__author__ = 'Debapriya Das'
__author_email__ = 'yodebu@gmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2018 Debapriya Das'


import sys
import WsseHeaders


def main():
    if (len(sys.argv) < 3):
        print("Usage: <USERNAME> <ORGANIZATION> <TOKEN>")
    else:
        a = WsseHeaders.WsseToken(sys.argv[0], sys.argv[1], sys.argv[2])
        headers = a.generateHeaders()
        return headers


if __name__ == "__main__":
    main()


import setuptools
import os

here = os.path.abspath(os.path.dirname(__file__))
long_description = ""
try:
    with open("README.md", "r") as fh:
        long_description = fh.read()
except:
    long_description = """
    WsseHeaders: WSSE for Python™
    ==========================


        WsseHeaders is the only WSSE Token generation library for Python, safe for human
        consumption.


        ``` {.sourceCode .python}
        >>> import WsseHeaders
        >>> r = WsseHeaders.WsseToken(username="yodebu", orgName="yodebu", token="base64tokenstring")
        >>> r.generateHeaders()
        UsernameToken Username="yodebu", PasswordDigest="EU4mrjk9tD3AVl3N1oJ6Yc+PA4k=", Nonce="e52165a00ae24d62bd44adb96f7c95dd", Created="2018-10-16T19:51:43+00:00", Organization="EXAMPLE_ORG"
        ```

        optional parameter: ``` pad ``` which is defaulted to ``` False``` and takes only Boolean values.

        ``` {.sourceCode .python}
        >>> import WsseHeaders
        >>> r = WsseHeaders.WsseToken(username="yodebu", orgName="yodebu", token="base64tokenstring", pad=True)
        >>> r.generateHeaders()
        UsernameToken Username="yodebu", PasswordDigest="EU4mrjk9tD3AVl3N1oJ6Yc+PA4k=", Nonce="e52165a00ae24d62bd44adb96f7c95dd", Created="2018-10-16T19:51:43+00:00", Organization="EXAMPLE_ORG"
        ```


        WsseHeaders officially supports Python 3.0 and above.

        Installation
        ------------

        To install WsseHeaders, simply use [pipenv](http://pipenv.org/) (or pip, of
        course):

        ``` {.sourceCode .bash}
        $ pipenv install WsseHeaders
        ✨🍰✨
        ```


        Documentation
        -------------

        Fantastic documentation to be available shortly at
        <http://docs.python.org/>, for a limited time only.

        How to Contribute
        -----------------

        1.  Check for open issues or open a fresh issue to start a discussion
            around a feature idea or a bug. 
        2.  Fork [the repository](https://github.com/light-bringer/wsse-headers) on
            GitHub to start making your changes to the **master** branch (or
            branch off of it).
        3.  Write a test which shows that the bug was fixed or that the feature
            works as expected.
        4.  Send a pull request and bug the maintainer until it gets merged and
            published. :) Make sure to add yourself to
            [AUTHORS](https://github.com/light-bringer/wsse-headers/blob/master/AUTHORS.rst).
    """

packages = ['WsseHeaders']

requires = [
    'pycrypto>=2.6.1',
    'pytz>=2018.5'
]

setuptools.setup(
    name="WsseHeaders",
    version="0.0.8",
    author="Debapriya Das",
    author_email="yodebu@gmail.com",
    description="A package to generate WSSE Headers",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='MIT',
    url="https://github.com/light-bringer/wsse-headers",
    # packages=setuptools.find_packages(),
    keywords='WSSE Headers generation',
    packages = packages,
    python_requires=">=3.0, <4",
    install_requires=requires,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development',
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)


# -*- coding: utf-8 -*-

from . import utilities as utils


class WsseToken():
    # _UserName = None
    # _Organization = None
    # _AuthToken = None
    # _DateString = None

    
    def getDateTime(self):
        return str(self.__DateString)
    

    def getOrg(self):
        return str(self.__Organization)
    

    def getUser(self):
        return str(self.__UserName)

    
    def __init__(self, username, orgName, token, pad=False):
        if username is None or username is "":
            raise ValueError
        elif orgName is None or orgName  is "":
            raise ValueError
        elif token is None or len(token) < 16:
            raise ValueError
            
        self.__UserName = username
        self.__Organization = orgName
        self.__AuthToken = token
        self.__DateString = utils.generateISOTimeString()
        key = utils.generateMD5(self.__UserName)
        iv = utils.decode_Base64(self.__AuthToken)
        base64iv = utils.base64.b64encode(iv.encode())
        AESObj = utils.AESCipher(key)
        self.__secret = AESObj.encrypt(token, base64iv, padding=pad)
        self.__nonce = utils.generate_nonce()[0]

    def generateHeaders(self):
        try:
            passworddigest = utils.generatePasswordDigest(
                self.__nonce, self.__DateString, self.__secret.decode('ASCII')
            )
            header = 'UsernameToken Username="%s", PasswordDigest="%s", Nonce="%s", Created="%s", Organization="%s"'%(
                self.__UserName,
                passworddigest, 
                self.__nonce , self.__DateString, 
                self.__Organization)
            return header
        except Exception as e:
            print(e)
            raise






# -*- coding: utf-8 -*-
'''
WsseHeaders.utilities
~~~~~~~~~~~~~~
This module provides utility functions that are used within WsseHeaders
that are also useful for external consumption.
Note : but not currently exposed for external consumption
'''

import datetime
import base64
import hashlib
import sys
from Crypto import Random
from Crypto.Cipher import AES
import uuid
import pytz

class AESCipher(object):
    '''
    AES Cipher Class for Python
    '''

    def __init__(self, key, BS=AES.block_size):
        '''
        input : key (key to encrypt), BS (Block Size)
        '''
        self.bs = BS
        self.key = key

    def encrypt(self, raw, iv, padding=False):
        print(iv)
        if padding:
            raw = self._pad(raw)
        try:
            cipher = AES.new(self.key, AES.MODE_CFB, iv)
            return base64.b64encode(iv + cipher.encrypt(raw))
        except Exception as e:
            print(e)
            raise

    def decrypt(self, enc):
        enc = base64.b64decode(enc)
        iv = enc[:AES.block_size]
        try:
            cipher = AES.new(self.key, AES.MODE_CBC, iv)
            return self._unpad(cipher.decrypt(enc[AES.block_size:])).decode('utf-8')
        except Exception as e:
            print(e)
            raise

    def _pad(self, s):
        return s + (self.bs - len(s) % self.bs) * chr(self.bs - len(s) % self.bs)

    @staticmethod
    def _unpad(s):
        return s[:-ord(s[len(s)-1:])]



def generateDateString(zone=datetime.timezone.utc):
    '''
    generate a datetime string
    input : None
    return : datetime string of UTC 
    '''
    dateformat = str('%Y%m%d%H%M%S')
    curDate = datetime.datetime.now(tz=zone).replace(microsecond=0)
    return curDate.strftime(dateformat)


def generateISOTimeString(zone='UTC'):
    '''
    generate a datetime string in ISO 8601 format
    input : None (default_timezone = 'UTC')
    return : datetime string in ISO 8601 format
    '''
    tz = pytz.timezone(zone)
    curdate = tz.localize(datetime.datetime.now().replace(microsecond=0))
    return curdate.isoformat()
    

def generateMD5(string, algo='md5'):
    '''
    Function to return a md5 String
    input : string, algo=md5
    return : md5hash(string)
    '''
    m = hashlib.new(algo)
    string = string.encode('utf-8')
    m.update(string)
    return m.hexdigest()


def decode_Base64(encoded_string):
    '''
    take a base64 string as <str> and decode it to original string as <str>
    input : encoded_string
    output : decoded_string
    '''
    b64str = encoded_string.encode()
    try:
        string = base64.b64decode(b64str).decode('utf-8')
    except Exception as e:
        sys.stdout.write("Exception: %s\n"%(str(e)))
        sys.stdout.flush()
        raise

    return string


def generate_nonce():
    '''
    Generate pseudo-random number and seconds since epoch (UTC)
    '''
    nonce = uuid.uuid4()
    oauth_timestamp, oauth_nonce = str(nonce.time), nonce.hex
    return oauth_nonce, oauth_timestamp


def generatePasswordDigest(nonce, timestamp, secret):
    concat_string = nonce + timestamp + secret
    try:
        sha1encoded = hashlib.sha1(concat_string.encode()).digest()
        encoded_sha1_string = base64.b64encode(sha1encoded)
        utf8encoded = encoded_sha1_string.decode('utf-8')
    except Exception as e:
        sys.stdout.write("Exception: %s\n"%(str(e)))
        sys.stdout.flush()
        raise
    return utf8encoded


